@@ -2,16 +2,15 @@
 `_renv_globals` <- new.env(parent = emptyenv())
 
 renv_global <- function(name, value) {
-  renv_global_get(name) %||% renv_global_set(name, value)
+  (`_renv_globals`[[name]]) %||% (`_renv_globals`[[name]] <- value)
 }
 
 renv_global_get <- function(name) {
-  if (exists(name, envir = `_renv_globals`, inherits = FALSE))
-    get(name, envir = `_renv_globals`, inherits = FALSE)
+  `_renv_globals`[[name]]
 }
 
 renv_global_set <- function(name, value) {
-  assign(name, value, envir = `_renv_globals`, inherits = FALSE)
+  `_renv_globals`[[name]] <- value
 }
 
 renv_global_clear <- function(name) {
